unit uebersi;

interface

uses objects,views,drivers,Dialogs,
     nwitype;

type
  PUSRadioButtons = ^TUSRadioButtons;
  TUSRadioButtons = object(TRadioButtons)
    procedure press(Item: Integer); virtual;
    {procedure moveto(Item: Integer); virtual;}
  end;

  PUSListBox = ^TUSListBox;
  TUSListBox = object(TListbox)
    procedure FocusItem(Item: Integer); virtual;
    procedure SelectItem(Item: Integer); virtual;
    function GetText(Item: Integer; MaxLen: Integer): String; virtual;
  end;

  PUSDialog = ^TUSDialog;
  TUSDialog = object(TDialog)
    DetailListBox : PUSListBox;
    DetailRadioButtons : PUSRadiobuttons;
    MasterListBox : PUSListBox;
    MasterRadiobuttons : PUSRadioButtons;

    constructor Init(var Bounds: TRect; ATitle: TTitleStr);
    destructor done; virtual;
    procedure HandleEvent(var Event: TEvent); virtual;
    procedure InitDialog;
    procedure UpdateDialog(Master:boolean);
    procedure Validate;
    procedure InsertItem;
    procedure DeleteItem;
    function ChooseFromList(coll:PNWColl):String;
  end;

procedure ShowUebersicht;

implementation

uses { Tvision: } app,msgbox,
     { ASNE:    } global,nwiproc,tvutil,auscalc,
     { Netz:    } nwBindry,nwfile;

Procedure ConvStr(var x:string); forward;

{ ************* USRadioButtons ********************* }

procedure TUSRadioButtons.press;
begin
  inherited press(item);
  message(owner,evBroadcast,cmRadioButtonsChanged,@self);
end;


{ ************* USListBox *************************** }

procedure TUSListBox.FocusItem(Item: Integer);
begin
  inherited FocusItem(Item);
  message(owner,evBroadCast,cmListChanged,@self);
end;

procedure TUSListBox.SelectItem(Item: Integer);
var s:string;
begin
  inherited SelectItem(Item);
  s:=PString(List^.at(item))^;
  s[1]:=char(not byte(s[1]) and $80 or byte(s[1]) and $7F);
  list^.atfree(item);
  list^.atinsert(item,newstr(s));
  drawview;
end;

function TUSListBox.GetText(Item: Integer; MaxLen: Integer): String;
var s:string;
begin
  s:=inherited GetText(Item,MaxLen-2);
  if s[1]>#128 then begin dec(s[1],128); s:=' '+s end
               else s:='  '+s;
  GetText:=s;
end;

{ ************* USDialog *************************** }

constructor TUSDialog.init;
begin
  inherited init(bounds,atitle);
  options := options or ofcentered;
  UebersichtActive:=true;
end;

destructor TUSDialog.done;
begin
  dispose(MasterListBox^.list,done);
  dispose(DetailListBox^.list,done);
  UebersichtActive:=false;
  inherited done;
end;

procedure TUSDialog.HandleEvent;

  procedure MyEvents(Event:TEvent);
  begin
    case event.what of
      evKeydown:
        case Event.keycode of
          kbIns: InsertItem;
          kbDel: DeleteItem;
          kbEnter: if MasterListBox^.getstate(sfFocused)
                      then DetailListBox^.focus;
          else exit;
        end;
      evBroadCast:
        case Event.keycode of
          cmRadioButtonsChanged:
              if Event.Infoptr = MasterRadioButtons
                then begin UpdateDialog(true); MasterListBox^.focus end
                else begin UpdateDialog(false); DetailListBox^.focus end;
          cmListChanged:
              if Event.Infoptr = MasterListBox
                then UpdateDialog(false);
          else exit;
        end;
      else exit;
    end;
    clearevent(event);
  end;

begin
  myevents(Event);
  inherited HandleEvent(Event);
end;

procedure TUSDialog.InitDialog;
var MColl:PNWColl;
    DColl:PNWColl;
begin
  MColl:=new(PNWColl,init(1,1));
  DColl:=new(PNWColl,init(1,1));
  MasterListBox^.newlist(Mcoll);
  DetailListBox^.newlist(Dcoll);
end;

procedure TUSDialog.UpdateDialog(Master:boolean);
var MColl:PNWColl;
    DColl:PNWColl;
    MasterStr,DetailStr,s:string;
    var i:byte;
begin
  if MasterRadiobuttons^.value = DetailRadiobuttons^.value  then
    begin
      if Master then
        with DetailRadiobuttons^ do
          case value of
            0: press(1);
            1: press(0);
          end
      else begin
        with MasterRadiobuttons^ do
          case value of
            0: press(1);
            1: press(0);
          end;
        exit;
      end;
    end;

  MColl:=PNWColl(MasterListBox^.list);
  DColl:=PNWColl(DetailListBox^.list);

  if Master then
    begin
      MColl^.freeall;
      case MasterRadioButtons^.value of
        0: GetObjNames('*',true,MColl^);
        1: GetObjNames('*',false,MColl^);
      end;
      MasterListBox^.setrange(MColl^.count);
    end;

  with MColl^ do
    if count>0 then MasterStr:=pstring(at(MasterListBox^.focused))^
               else MasterStr:='*';
  with DColl^ do
    if count>0 then DetailStr:=pstring(at(DetailListBox^.focused))^
               else DetailStr:='*';
  ConvStr(Masterstr);
  ConvStr(DetailStr);
  DColl^.freeall;
  case MasterRadioButtons^.value *16 + DetailRadioButtons^.value of
    $01: GetRelationObjects(MasterStr,OT_USER_GROUP,'GROUPS_I''M_IN',DColl^);
    $10: GetRelationObjects(MasterStr,OT_USER,'GROUP_MEMBERS',DColl^);
    $02: for i:=0 to 31 do GetUserTrustees(i,Name2ID(MasterStr,OT_USER_GROUP),DColl^);
    $12: for i:=0 to 31 do GetUserTrustees(i,Name2ID(MasterStr,OT_USER),DColl^);
  end;

  DetailListBox^.setrange(DColl^.count);
  redraw;
end;

procedure TUSDialog.Validate;
var typ:byte;
    value,prop,name:string;

  function Matches(Item: Pointer): Boolean; far;
  begin
    Matches:=(PAusnahme(Item)^.equal(typ,name,value,prop)=1);
  end;

var i:word;
begin
  for i:=0 to MasterListBox^.list^.count-1 do
    case MasterRadioButtons^.value *16 + DetailRadioButtons^.value of
    1:;
    end;
end;

procedure TUSDialog.InsertItem;
var coll:TNWColl;
    MasterStr,s:String;
    rights:word;
begin
  if MasterListBox^.List^.count > 0 then
    MasterStr:=pstring(MasterListBox^.list^.at(MasterListBox^.focused))^;
  ConvStr(Masterstr);
  coll.init(1,1);
  if MasterListBox^.getstate(sffocused) then
    begin
      case MasterRadioButtons^.value of
        0: ; { Gruppe erzeugen }
        1: ; { User erzeugen }
      end;
      updatedialog(True);
    end;
  if DetailListBox^.getstate(sffocused) then
    begin
      case MasterRadioButtons^.value of
        0: case DetailRadioButtons^.value of  { Master: Gruppe }
             1: begin { Gruppe: User aufnehmen }
                  getObjNames('*',false,coll);
                  s:=ChooseFromList(@coll);
                  if s<>'' then AddUserToGroup(s,MasterStr);
                end;
             2: if gettrusteebox(s,rights) then { Rechte fr Gruppe }
                  settrustee(0,s,name2id(Masterstr,OT_USER_GROUP),rights);
           end;
        1: case DetailRadioButtons^.value of  { Master: User }
             0: begin { User in Gruppe }
                  getObjNames('*',true,coll);
                  s:=ChooseFromList(@coll);
                  if s<>'' then AddUserToGroup(MasterStr,s);
                end;
             2: if gettrusteebox(s,rights) then { Rechte fr User }
                  settrustee(0,s,name2id(Masterstr,OT_USER),rights);
           end;
      end;
      updatedialog(false);
    end;
  coll.done;
end;

procedure TUSDialog.DeleteItem;
var coll:TNWColl;
    MasterStr,DetailStr,s:String;
begin
  if MasterListBox^.List^.count > 0 then
    MasterStr:=pstring(MasterListBox^.list^.at(MasterListBox^.focused))^
    else exit;
  if DetailListBox^.List^.count > 0 then
    DetailStr:=pstring(DetailListBox^.list^.at(DetailListBox^.focused))^
    else DetailStr:='';
  ConvStr(Masterstr);
  ConvStr(DetailStr);

  coll.init(1,1);
  if MasterListBox^.getstate(sffocused) then
    begin
      case MasterRadioButtons^.value of
        0: ; { Gruppe lschen }
        1: ; { User lschen }
      end;
      updatedialog(True);
    end;
  if DetailListBox^.getstate(sffocused) then
    begin
      if DetailStr='' then exit;
      case MasterRadioButtons^.value of
        0: case DetailRadioButtons^.value of  { Master: Gruppe }
             1: begin { Gruppe: User lschen }
                  if messagebox('Lsche '+DetailStr+' aus Gruppe '+
                                MasterStr+'?',nil,mfOkCancel) = cmOk
                    then DeleteUserFromGroup(DetailStr,MasterStr);
                end;
             2:  { Rechte fr Gruppe }
                if messagebox('Lsche Rechte'^m+DetailStr+^m+
                              'der Gruppe '+MasterStr+'?',nil,mfOkCancel) = cmOk
                    then begin
                      system.delete(detailstr,1,11);
                      deletetrustee(0,detailstr,name2id(Masterstr,OT_USER_GROUP));
                    end;
           end;
        1: case DetailRadioButtons^.value of  { Master: User }
             0: begin { User aus Gruppe }
                  if messagebox('Lsche '+MasterStr+' aus Gruppe '+
                                DetailStr+'?',nil,mfOkCancel) = cmOk
                    then DeleteUserFromGroup(MasterStr,DetailStr);
                end;
             2:  { Rechte fr User }
                if messagebox('Lsche Rechte'^m+DetailStr+^m+
                              'des Users '+MasterStr+'?',nil,mfOkCancel) = cmOk
                    then begin
                      system.delete(detailstr,1,11);
                      deletetrustee(0,DetailStr,name2id(Masterstr,OT_USER));
                    end;
           end;
      end;
      updatedialog(false);
    end;
  coll.done;
end;

function TUSDialog.ChooseFromList(coll:PNWColl):String;
{$I listbox.src }
var d:PDialog;
    i:word;
    s:string;
begin
  Datarec.listbox.ps:=PStringCollection(coll);
  Datarec.listbox.Focused:=0;
  d:=makedialog;
  if application^.executedialog(d,@datarec) = cmOk
    then ChooseFromList:=Pstring(datarec.Listbox.PS^.at(datarec.ListBox.focused))^
    else ChooseFromList:='';
end;


{ ************* Proceduren ************************* }

Procedure ConvStr(var x:string);
begin
  x[1]:=char(byte(x[1]) and $7F);
end;

procedure ShowUebersicht;
{$I uebersi.src }
var d:PUSDialog;
begin
  d:=makedialog;
  d^.InitDialog;
  d^.UpdateDialog(true);
  d^.MasterListBox^.focus;
  desktop^.insert(d);
end;

end.